@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2020 The Project Lombok Authors.
+ * Copyright (C) 2016-2021 The Project Lombok Authors.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@ import static lombok.javac.JavacTreeMaker.TypeTag.typeTag;
 import java.io.IOException;
 import java.io.Writer;
 import java.lang.reflect.Field;
-import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -279,7 +278,6 @@ public class PrettyPrinter extends JCTree.Visitor {
 		aligned = false;
 	}
 	
-	
 	private void println() {
 		try {
 			out.write(LINE_SEP);
@@ -516,10 +514,12 @@ public class PrettyPrinter extends JCTree.Visitor {
 		boolean isInterface = (tree.mods.flags & INTERFACE) != 0;
 		boolean isAnnotationInterface = isInterface && (tree.mods.flags & ANNOTATION) != 0;
 		boolean isEnum = (tree.mods.flags & ENUM) != 0;
+		boolean isRecord = (tree.mods.flags & RECORD) != 0;
 		
 		if (isAnnotationInterface) print("@interface ");
 		else if (isInterface) print("interface ");
 		else if (isEnum) print("enum ");
+		else if (isRecord) print("record ");
 		else print("class ");
 		
 		print(tree.name);
@@ -531,6 +531,9 @@ public class PrettyPrinter extends JCTree.Visitor {
 			print(tree.typarams, ", ");
 			print(">");
 		}
+		
+		if (isRecord) printRecordConstructor(tree.defs);
+		
 		JCTree extendsClause = getExtendsClause(tree);
 		if (extendsClause != null) {
 			print(" extends ");
@@ -542,6 +545,12 @@ public class PrettyPrinter extends JCTree.Visitor {
 			print(tree.implementing, ", ");
 		}
 		
+		List<JCExpression> permitting = readObject(tree, "permitting", List.<JCExpression>nil());
+		if (permitting.nonEmpty()) {
+			print(" permits ");
+			print(permitting, ", ");
+		}
+		
 		println(" {");
 		indent++;
 		printClassMembers(tree.defs, isEnum, isInterface);
@@ -551,6 +560,23 @@ public class PrettyPrinter extends JCTree.Visitor {
 		currentTypeName = prevTypeName;
 	}
 	
+	private void printRecordConstructor(List<JCTree> members) {
+		boolean first = true;
+		print("(");
+		for (JCTree member : members) {
+			if (member instanceof JCVariableDecl) {
+			JCVariableDecl variableDecl = (JCVariableDecl) member;
+				if ((variableDecl.mods.flags & GENERATED_MEMBER) != 0) {
+					if (!first) print(", ");
+					first = false;
+					printAnnotations(variableDecl.mods.annotations, false);
+					printVarDef0(variableDecl);
+				}
+			}
+		}
+		print(")");
+	}
+	
 	private void printClassMembers(List<JCTree> members, boolean isEnum, boolean isInterface) {
 		Class<?> prefType = null;
 		int typeOfPrevEnumMember = isEnum ? 3 : 0; // 1 = normal, 2 = with body, 3 = no enum field yet.
@@ -580,6 +606,7 @@ public class PrettyPrinter extends JCTree.Visitor {
 				JCTree init = ((JCVariableDecl) member).init;
 				typeOfPrevEnumMember = init instanceof JCNewClass && ((JCNewClass) init).def != null ? 2 : 1;
 			} else if (member instanceof JCVariableDecl) {
+				if ((((JCVariableDecl) member).mods.flags & GENERATED_MEMBER) != 0) continue;
 				if (prefType != null && prefType != JCVariableDecl.class) println();
 				if (isInterface) flagMod = -1L & ~(PUBLIC | STATIC | FINAL);
 				print(member);
@@ -792,22 +819,29 @@ public class PrettyPrinter extends JCTree.Visitor {
 			print(tree.name);
 		}
 		
+		boolean argsLessConstructor = false;
+		if (isConstructor && (tree.mods.flags & COMPACT_RECORD_CONSTRUCTOR) != 0) {
+			argsLessConstructor = true;
+		}
+		
 		boolean first = true;
-		print("(");
-		
-		JCVariableDecl recvparam = readObject(tree, "recvparam", null);
-		if (recvparam != null) {
-			printVarDefInline(recvparam);
-			first = false;
+		if (!argsLessConstructor) {
+			print("(");
+			
+			JCVariableDecl recvparam = readObject(tree, "recvparam", null);
+			if (recvparam != null) {
+				printVarDefInline(recvparam);
+				first = false;
+			}
+			
+			for (JCVariableDecl param : tree.params) {
+				if (!first) print(", ");
+				first = false;
+				printVarDefInline(param);
+			}
+			print(")");
 		}
 		
-		for (JCVariableDecl param : tree.params) {
-			if (!first) print(", ");
-			first = false;
-			printVarDefInline(param);
-		}
-		print(")");
-		
 		if (tree.thrown.nonEmpty()) {
 			print(" throws ");
 			print(tree.thrown, ", ");
@@ -988,6 +1022,8 @@ public class PrettyPrinter extends JCTree.Visitor {
 		if ((v & TRANSIENT) != 0) print("transient ");
 		if ((v & NATIVE) != 0) print("native ");
 		if ((v & ABSTRACT) != 0) print("abstract ");
+		if ((v & SEALED) != 0) print("sealed ");
+		if ((v & NON_SEALED) != 0) print("non-sealed ");
 		if ((v & STRICTFP) != 0) print("strictfp ");
 		if ((v & DEFAULT) != 0 && (v & INTERFACE) == 0) print("default ");
 	}
@@ -1296,13 +1332,16 @@ public class PrettyPrinter extends JCTree.Visitor {
 	@Override public void visitCase(JCCase tree) {
 		// Starting with JDK12, switches allow multiple expressions per case, and can take the form of an expression (preview feature).
 		
-		List<JCExpression> pats = readObject(tree, "pats", null); // JDK 12+
+		List<JCTree> pats = readObject(tree, "labels", null); // JDK 17+
 		if (pats == null) {
-			JCExpression pat = readObject(tree, "pat", null); // JDK -11
-			pats = pat == null ? List.<JCExpression>nil() : List.of(pat);
+			pats = readObject(tree, "pats", null); // JDK 12-17
+		}
+		if (pats == null) {
+			JCTree pat = readObject(tree, "pat", null); // JDK -11
+			pats = pat == null ? List.<JCTree>nil() : List.of(pat);
 		}
 		
-		if (pats.isEmpty()) {
+		if (pats.isEmpty() || pats.size() == 1 && pats.head.getClass().getName().endsWith("$JCDefaultCaseLabel")) {
 			aPrint("default");
 		} else {
 			aPrint("case ");
@@ -1387,9 +1426,26 @@ public class PrettyPrinter extends JCTree.Visitor {
 	}
 	
 	void printBindingPattern(JCTree tree) {
-		print((JCExpression) readObject(tree, "vartype", null));
+		JCTree var = readObject(tree, "var", tree);
+		print((JCExpression) readObject(var, "vartype", null));
 		print(" ");
-		print((Name) readObject(tree, "name", null));
+		print((Name) readObject(var, "name", null));
+	}
+	
+	void printDefaultCase(JCTree tree) {
+		print("default");
+	}
+	
+	void printGuardPattern(JCTree tree) {
+		print((JCTree) readObject(tree, "patt", null));
+		print(" && ");
+		print((JCExpression) readObject(tree, "expr", null));
+	}
+	
+	void printParenthesizedPattern(JCTree tree) {
+		print("(");
+		print((JCTree) readObject(tree, "pattern", null));
+		print(")");
 	}
 	
 	@Override public void visitTry(JCTry tree) {
@@ -1610,6 +1666,12 @@ public class PrettyPrinter extends JCTree.Visitor {
 			printYieldExpression(tree);
 		} else if (className.endsWith("$JCBindingPattern")) { // Introduced as preview in JDK14
 			printBindingPattern(tree);
+		} else if (className.endsWith("$JCDefaultCaseLabel")) { // Introduced in JDK17
+			printDefaultCase(tree);
+		} else if (className.endsWith("$JCGuardPattern")) { // Introduced in JDK17
+			printGuardPattern(tree);
+		} else if (className.endsWith("$JCParenthesizedPattern")) { // Introduced in JDK17
+			printParenthesizedPattern(tree);
 		} else {
 			throw new AssertionError("Unhandled tree type: " + tree.getClass() + ": " + tree);
 		}
