@@ -27,16 +27,25 @@ class BuilderWithToBuilder<T> {
 		@java.lang.SuppressWarnings("all")
 		BuilderWithToBuilderBuilder() {
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> one(final String one) {
 			this.one = one;
 			return this;
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> two(final String two) {
 			this.two = two;
 			return this;
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> foo(final T foo) {
 			this.foo = foo;
@@ -114,16 +123,25 @@ class ConstructorWithToBuilder<T> {
 		@java.lang.SuppressWarnings("all")
 		ConstructorWithToBuilderBuilder() {
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> mOne(final String mOne) {
 			this.mOne = mOne;
 			return this;
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> baz(final T baz) {
 			this.baz = baz;
 			return this;
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> bars(final com.google.common.collect.ImmutableList<T> bars) {
 			this.bars = bars;
@@ -147,4 +165,54 @@ class ConstructorWithToBuilder<T> {
 	public ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> toBuilder() {
 		return new ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T>().mOne(this.mOne).baz(this.foo).bars(this.bars);
 	}
+}
+class StaticMethodWithToBuilder<T> {
+	private T foo;
+	
+	public StaticMethodWithToBuilder(T foo) {
+		this.foo = foo;
+	}
+
+	public static <T> StaticMethodWithToBuilder<T> of(T foo) {
+		return new StaticMethodWithToBuilder<T>(foo);
+	}
+
+	@java.lang.SuppressWarnings("all")
+	public static class StaticMethodWithToBuilderBuilder<T> {
+		@java.lang.SuppressWarnings("all")
+		private T foo;
+
+		@java.lang.SuppressWarnings("all")
+		StaticMethodWithToBuilderBuilder() {
+		}
+		/**
+		 * @return {@code this}.
+		 */
+		@java.lang.SuppressWarnings("all")
+		public StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> foo(final T foo) {
+			this.foo = foo;
+			return this;
+		}
+
+		@java.lang.SuppressWarnings("all")
+		public StaticMethodWithToBuilder<T> build() {
+			return StaticMethodWithToBuilder.<T>of(this.foo);
+		}
+
+		@java.lang.Override
+		@java.lang.SuppressWarnings("all")
+		public java.lang.String toString() {
+			return "StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder(foo=" + this.foo + ")";
+		}
+	}
+
+	@java.lang.SuppressWarnings("all")
+	public static <T> StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> builder() {
+		return new StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T>();
+	}
+
+	@java.lang.SuppressWarnings("all")
+	public StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> toBuilder() {
+		return new StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T>().foo(this.foo);
+	}
 }
\ No newline at end of file
