@@ -1,3 +1,5 @@
+// skip-idempotent
+//version 8:
 class CheckerFrameworkBasic {
 	private final int x;
 	private final int y;
@@ -20,9 +22,8 @@ class CheckerFrameworkBasic {
 	/**
 	 * @return {@code this}.
 	 */
-	@org.checkerframework.common.returnsreceiver.qual.This
 	@java.lang.SuppressWarnings("all")
-	public CheckerFrameworkBasic setZ(final int z) {
+	public @org.checkerframework.common.returnsreceiver.qual.This CheckerFrameworkBasic setZ(final int z) {
 		this.z = z;
 		return this;
 	}
