@@ -15,7 +15,7 @@ class ConflictingStaticConstructorNames {
 	@java.lang.Override
 	@java.lang.SuppressWarnings("all")
 	public int hashCode() {
-		int result = 1;
+		final int result = 1;
 		return result;
 	}
 	@java.lang.Override
