@@ -2,7 +2,7 @@ class GetterLazy {
 	static class ValueType {
 	}
 	private final java.util.concurrent.atomic.AtomicReference<java.lang.Object> fieldName = new java.util.concurrent.atomic.AtomicReference<java.lang.Object>();
-	@java.lang.SuppressWarnings("all")
+	@java.lang.SuppressWarnings({"all", "unchecked"})
 	public ValueType getFieldName() {
 		java.lang.Object value = this.fieldName.get();
 		if (value == null) {
