@@ -10,3 +10,8 @@ class LoggerCommonsWithDifferentName {
 	@java.lang.SuppressWarnings("all")
 	private static final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog("DifferentName");
 }
+class LoggerCommonsWithStaticField {
+	@java.lang.SuppressWarnings("all")
+	private static final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(LoggerCommonsWithStaticField.TOPIC);
+	static final String TOPIC = "StaticField";
+}
\ No newline at end of file
