@@ -14,4 +14,17 @@ class LoggerJulWithStaticField {
 	@java.lang.SuppressWarnings("all")
 	private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(LoggerJulWithStaticField.TOPIC);
 	static final String TOPIC = "StaticField";
+}
+enum LoggerJulWithEnum {
+	CONSTANT;
+	@java.lang.SuppressWarnings("all")
+	private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(LoggerJulWithEnum.class.getName());
+}
+class LoggerJulWithInnerEnum {
+
+	enum Inner {
+		CONSTANT;
+		@java.lang.SuppressWarnings("all")
+		private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(Inner.class.getName());
+	}
 }
\ No newline at end of file
