@@ -1,3 +1,4 @@
+//version 8:
 class LoggerLog4j2 {
 	@java.lang.SuppressWarnings("all")
 	private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(LoggerLog4j2.class);
@@ -14,4 +15,17 @@ class LoggerLog4j2WithStaticField {
 	@java.lang.SuppressWarnings("all")
 	private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(LoggerLog4j2WithStaticField.TOPIC);
 	static final String TOPIC = "StaticField";
+}
+enum LoggerLog4j2WithEnum {
+	CONSTANT;
+	@java.lang.SuppressWarnings("all")
+	private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(LoggerLog4j2WithEnum.class);
+}
+class LoggerLog4j2WithInnerEnum {
+
+	enum Inner {
+		CONSTANT;
+		@java.lang.SuppressWarnings("all")
+		private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(Inner.class);
+	}
 }
\ No newline at end of file
