@@ -10,3 +10,8 @@ class LoggerXSlf4jWithDifferentName {
 	@java.lang.SuppressWarnings("all")
 	private static final org.slf4j.ext.XLogger log = org.slf4j.ext.XLoggerFactory.getXLogger("DifferentName");
 }
+class LoggerXSlf4jWithStaticField {
+	@java.lang.SuppressWarnings("all")
+	private static final org.slf4j.ext.XLogger log = org.slf4j.ext.XLoggerFactory.getXLogger(LoggerXSlf4jWithStaticField.TOPIC);
+	static final String TOPIC = "StaticField";
+}
\ No newline at end of file
