@@ -2,10 +2,15 @@ final class UtilityClass {
 	private static long someField = System.currentTimeMillis();
 	static void someMethod() {
 		System.out.println();
+		new InnerClass();
+		new InnerStaticClass();
 	}
 	protected static class InnerClass {
 		private String innerInnerMember;
 	}
+	protected static class InnerStaticClass {
+		private String innerInnerMember;
+	}
 	@java.lang.SuppressWarnings("all")
 	private UtilityClass() {
 		throw new java.lang.UnsupportedOperationException("This is a utility class and cannot be instantiated");
