@@ -14,7 +14,7 @@ final class ValueStaticField {
 	@java.lang.Override
 	@java.lang.SuppressWarnings("all")
 	public int hashCode() {
-		int result = 1;
+		final int result = 1;
 		return result;
 	}
 	@java.lang.Override
