@@ -9,14 +9,23 @@ import lombok.Builder;
     @java.lang.SuppressWarnings("all") BuilderWithToBuilderBuilder() {
       super();
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> one(final String one) {
       this.one = one;
       return this;
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> two(final String two) {
       this.two = two;
       return this;
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") BuilderWithToBuilder.BuilderWithToBuilderBuilder<T> foo(final T foo) {
       this.foo = foo;
       return this;
@@ -93,14 +102,23 @@ import lombok.Builder;
     @java.lang.SuppressWarnings("all") ConstructorWithToBuilderBuilder() {
       super();
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> mOne(final String mOne) {
       this.mOne = mOne;
       return this;
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> baz(final T baz) {
       this.baz = baz;
       return this;
     }
+    /**
+     * @return {@code this}.
+     */
     public @java.lang.SuppressWarnings("all") ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T> bars(final com.google.common.collect.ImmutableList<T> bars) {
       this.bars = bars;
       return this;
@@ -126,3 +144,38 @@ import lombok.Builder;
     return new ConstructorWithToBuilder.ConstructorWithToBuilderBuilder<T>().mOne(this.mOne).baz(this.foo).bars(this.bars);
   }
 }
+class StaticMethodWithToBuilder<T> {
+  public static @java.lang.SuppressWarnings("all") class StaticMethodWithToBuilderBuilder<T> {
+    private @java.lang.SuppressWarnings("all") T foo;
+    @java.lang.SuppressWarnings("all") StaticMethodWithToBuilderBuilder() {
+      super();
+    }
+    /**
+     * @return {@code this}.
+     */
+    public @java.lang.SuppressWarnings("all") StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> foo(final T foo) {
+      this.foo = foo;
+      return this;
+    }
+    public @java.lang.SuppressWarnings("all") StaticMethodWithToBuilder<T> build() {
+      return StaticMethodWithToBuilder.<T>of(this.foo);
+    }
+    public @java.lang.Override @java.lang.SuppressWarnings("all") java.lang.String toString() {
+      return (("StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder(foo=" + this.foo) + ")");
+    }
+  }
+  private T foo;
+  public StaticMethodWithToBuilder(T foo) {
+    super();
+    this.foo = foo;
+  }
+  public static @Builder(toBuilder = true) <T>StaticMethodWithToBuilder<T> of(T foo) {
+    return new StaticMethodWithToBuilder<T>(foo);
+  }
+  public static @java.lang.SuppressWarnings("all") <T>StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> builder() {
+    return new StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T>();
+  }
+  public @java.lang.SuppressWarnings("all") StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T> toBuilder() {
+    return new StaticMethodWithToBuilder.StaticMethodWithToBuilderBuilder<T>().foo(this.foo);
+  }
+}
\ No newline at end of file
