@@ -22,4 +22,13 @@ import lombok.extern.apachecommons.CommonsLog;
   LoggerCommonsWithDifferentName() {
     super();
   }
+}
+@CommonsLog(topic = LoggerCommonsWithStaticField.TOPIC) class LoggerCommonsWithStaticField {
+  private static final org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(LoggerCommonsWithStaticField.TOPIC);
+  static final String TOPIC = "StaticField";
+  <clinit>() {
+  }
+  LoggerCommonsWithStaticField() {
+    super();
+  }
 }
\ No newline at end of file
