@@ -28,6 +28,29 @@ class LoggerJBossLogOuter {
     super();
   }
 }
+@JBossLog enum LoggerJBossLogWithEnum {
+  CONSTANT(),
+  private static final org.jboss.logging.Logger log = org.jboss.logging.Logger.getLogger(LoggerJBossLogWithEnum.class);
+  <clinit>() {
+  }
+  LoggerJBossLogWithEnum() {
+    super();
+  }
+}
+class LoggerJBossLogWithInnerEnum {
+  @JBossLog enum Inner {
+    CONSTANT(),
+    private static final org.jboss.logging.Logger log = org.jboss.logging.Logger.getLogger(Inner.class);
+    <clinit>() {
+    }
+    Inner() {
+      super();
+    }
+  }
+  LoggerJBossLogWithInnerEnum() {
+    super();
+  }
+}
 @JBossLog(topic = "DifferentLogger") class LoggerJBossLogWithDifferentLoggerName {
   private static final org.jboss.logging.Logger log = org.jboss.logging.Logger.getLogger("DifferentLogger");
   <clinit>() {
