@@ -31,4 +31,27 @@ import lombok.extern.java.Log;
   LoggerJulWithStaticField() {
     super();
   }
+}
+@Log enum LoggerJulWithEnum {
+  CONSTANT(),
+  private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(LoggerJulWithEnum.class.getName());
+  <clinit>() {
+  }
+  LoggerJulWithEnum() {
+    super();
+  }
+}
+class LoggerJulWithInnerEnum {
+  @Log enum Inner {
+    CONSTANT(),
+    private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(Inner.class.getName());
+    <clinit>() {
+    }
+    Inner() {
+      super();
+    }
+  }
+  LoggerJulWithInnerEnum() {
+    super();
+  }
 }
\ No newline at end of file
