@@ -31,4 +31,27 @@ import lombok.extern.log4j.Log4j2;
   LoggerLog4j2WithStaticField() {
     super();
   }
+}
+@Log4j2 enum LoggerLog4j2WithEnum {
+  CONSTANT(),
+  private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(LoggerLog4j2WithEnum.class);
+  <clinit>() {
+  }
+  LoggerLog4j2WithEnum() {
+    super();
+  }
+}
+class LoggerLog4j2WithInnerEnum {
+  @Log4j2 enum Inner {
+    CONSTANT(),
+    private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(Inner.class);
+    <clinit>() {
+    }
+    Inner() {
+      super();
+    }
+  }
+  LoggerLog4j2WithInnerEnum() {
+    super();
+  }
 }
\ No newline at end of file
