@@ -6,14 +6,14 @@ public class NonNullWithAlternateException {
   public void testMethod(@lombok.NonNull String arg) {
     if ((arg == null))
         {
-          throw new java.lang.IllegalArgumentException("arg is null");
+          throw new java.lang.IllegalArgumentException("arg is marked non-null but is null");
         }
     System.out.println(arg);
   }
   public @java.lang.SuppressWarnings("all") void setTest(final @lombok.NonNull String test) {
     if ((test == null))
         {
-          throw new java.lang.IllegalArgumentException("test is null");
+          throw new java.lang.IllegalArgumentException("test is marked non-null but is null");
         }
     this.test = test;
   }
