@@ -5,11 +5,18 @@ public class OnXJava8StyleOn7 {
   @interface Bar {
     String stuff() default "";
   }
+  @interface Array {
+    String[] value() default {};
+  }
   @lombok.Getter() String a;
   @lombok.Setter() String b;
   @lombok.Setter() String c;
   @lombok.Setter() String d;
   @lombok.Getter() String e;
+  @lombok.Getter() String f;
+  @lombok.Getter() String g;
+  @lombok.Getter() String h;
+  @lombok.Getter() String i;
   public OnXJava8StyleOn7() {
     super();
   }
@@ -28,4 +35,16 @@ public class OnXJava8StyleOn7 {
   public @Foo(value = "c") @Bar(stuff = "d") @java.lang.SuppressWarnings("all") String getE() {
     return this.e;
   }
+  public @Array @java.lang.SuppressWarnings("all") String getF() {
+    return this.f;
+  }
+  public @Array() @java.lang.SuppressWarnings("all") String getG() {
+    return this.g;
+  }
+  public @Array({}) @java.lang.SuppressWarnings("all") String getH() {
+    return this.h;
+  }
+  public @Array({"a", "b"}) @java.lang.SuppressWarnings("all") String getI() {
+    return this.i;
+  }
 }
