@@ -11,3 +11,8 @@ class LoggerCommonsWithImport {
 @CommonsLog(topic="DifferentName")
 class LoggerCommonsWithDifferentName {
 }
+
+@CommonsLog(topic=LoggerCommonsWithStaticField.TOPIC)
+class LoggerCommonsWithStaticField {
+	static final String TOPIC = "StaticField";
+}
\ No newline at end of file
