@@ -1,3 +1,4 @@
+//version 8:
 import lombok.extern.log4j.Log4j2;
 
 @lombok.extern.log4j.Log4j2
@@ -15,4 +16,16 @@ class LoggerLog4j2WithDifferentName {
 @Log4j2(topic=LoggerLog4j2WithStaticField.TOPIC)
 class LoggerLog4j2WithStaticField {
 	static final String TOPIC = "StaticField";
+}
+
+@Log4j2
+enum LoggerLog4j2WithEnum {
+	CONSTANT;
+}
+
+class LoggerLog4j2WithInnerEnum {
+	@Log4j2
+	enum Inner {
+		CONSTANT;
+	}
 }
\ No newline at end of file
