@@ -8,6 +8,18 @@ class LoggerSlf4j {
 class LoggerSlf4jWithImport {
 }
 
+@Slf4j
+enum LoggerSlf4jWithEnum {
+	CONSTANT;
+}
+
+class LoggerSlf4jWithInnerEnum {
+	@Slf4j
+	enum Inner {
+		CONSTANT;
+	}
+}
+
 class LoggerSlf4jOuter {
 	@lombok.extern.slf4j.Slf4j
 	static class Inner {
